#ifndef __IUF_GAMEOBJ
#define __IUF_GAMEOBJ
#include <math.h>
#include <vcg/Point3.h>
#ifndef M_PI
#define M_PI 3.14159265358979323846
#endif

/** This class represent a generic game object, with its speed, position etc.. 
This class is the base class  that is used to derive all the objects represented 
in the game: asteroids, ships bullets explosion and so on. 
Each GameObj has a specific position in the game space, a 
velocity vector, a heading (direction) angle (that can be 
obviously different from the velocity direction), a main 
rotation axis and an angular speed (think of asteroids inertia).*/
using namespace vcg ;
class GameObj
{
 public:
	/// Default constructor
	GameObj();
	/// constructor with given position and direction
	GameObj(const Point3f &_p, const float _angle);
	/// constructor with given position and direction and velocity
	GameObj(const Point3f &_p, const float _angle, const Point3f &_v);
	/// Position of a generic object in the game
	Point3f p;	  
	/// Rotation Axis 
	Point3f axis;	  
	///Direction (in degree)
	float angle;  
	/// Speed Component (mt / sec)  
	Point3f v;  
	/// Angular Speed (deg / sec)
	float av;
	/// If the object is active or not
	bool active; 

	/** Basic Redraw Function it does nothing.
	All derived classes should provide their own Draw functions.
	*/
	virtual void Draw(){}
	/** Basic update function.
	It updates the position and the direction of the object according its current 
	speed and the elapsed time (expressed as millisec).
	*/
	virtual void Update(int t)
		{
			p += (t/1000.0f*v);
			angle += (t/1000.0f*av);
		};
	/** Return the unitary direction vector.
	*/

	Point3f VDir() {
		return Point3f(cos(M_PI*angle/180.0),
									 sin(M_PI*angle/180.0),
									 0);}; 
};

/*----------------------------------*/
/** This class represent an explosion */
class Exp : public GameObj
{
public:
	int ttl;
	void Draw();
	void Update(int t);
};
#endif // ifndef __IUF_GAMEOBJ